import { ConflictException, Injectable } from '@nestjs/common'
import { MoneyUpdateType } from '@prisma/client'
import { PrismaService } from 'src/prisma/prisma.service'

@Injectable()
export class GameService {
  constructor (private prisma: PrismaService) {}

  async find() {
    return await this.prisma.game.findUnique({ where: { id: 1 } })
  }

  async start() {
    const game = await this.find()
    if (game.started)
      throw new ConflictException("La partie a déjà démarré.")
    const players = await this.prisma.player.findMany()
    const alreadyStarted = game.currentRunnerId !== null
    if (!alreadyStarted) {
      for (const player of players) {
        await this.prisma.moneyUpdate.create({
          data: {
            playerId: player.id,
            amount: 2000,
            reason: MoneyUpdateType.START,
          }
        })
      }
    }
    const runnerId = alreadyStarted ? game.currentRunnerId : players[Math.trunc(players.length * Math.random())].id
    return await this.prisma.game.update({
      where: { id: 1 },
      data: {
        started: true,
        currentRunnerId: runnerId,
      },
    })
  }

  async stop() {
    const game = await this.find()
    if (!game.started)
      throw new ConflictException("La partie n'a pas encore démarré.")
    return await this.prisma.game.update({
      where: { id: 1 },
      data: {
        started: false,
      },
    })
  }

  async reset() {
    await this.prisma.moneyUpdate.deleteMany()
    await this.prisma.challengeAction.deleteMany()
    await this.prisma.trainTrip.deleteMany()
    await this.prisma.geolocation.deleteMany()
    await this.prisma.player.updateMany({ data: { money: 0 } })
    await this.prisma.game.update({
      where: { id: 1 },
      data: {
        started: false,
        currentRunnerId: null,
      },
    })
    return await this.find()
  }
}
